"""
Computation of transcendental functions.
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.pari.py_pari as pari
import sage.rings.complex_number

def __prep_num(x):
    if isinstance(x, sage.rings.complex_number.ComplexNumber):
        x = str(x).replace("i","I")
    else:
        x = str(x)
    return x

I = sage.rings.complex_field.ComplexField().gen(0)
def __eval(x):
    return eval(x)

def gamma(s):
    """
    Gamma function at s.
    """
    return pari.pari(s).gamma().python()

def gamma_inc(s, t):
    """
    Incomplete Gamma function Gamma(s,t).

    WARNING: This function requires having Mathematica installed on
    your computer.  If you are reading this and know a reference for
    how to compute Gamma(s,t), let me know so I (or you) can implement
    it in SAGE.   (Pari has the incomplete Gamma function, but only
    for s and t real.)
    
    """
    try:
        import sage.interface.mathematica as M
        m = M.Math()
    except:
        raise RuntimeError, "You must have mathematica installed to use gamma_inc."
    t = "Gamma[%s, %s]*1.0"%(s,t)
    t = t.replace("e","*10^")
    s = m.cmd(t)
    return M.eval_num(s)
    

def zeta(s):
    """
    Riemann zeta function at s with s a real or complex number.
    EXAMPLES:
        >> zeta(2)
        1.6449340668482264364724151667
    """
    return pari.pari(s).zeta().python()

def zeta_symmetric(s):
    """
    Completed function xi(s) that satisfies xi(s) = xi(1-s) and
    has zeros at the same points as the Riemann zeta function.

    More precisely, xi(s) = gamma(s/2 + 1) * (s-1) * pi^(-s/2) * zeta(s)
    EXAMPLES:
        >> zeta_symmetric(0.7)
        0.49758041465112690357779107524
        >> zeta_symmetric(1-0.7)
        0.49758041465112690357779107525

    REFERENCE:
      I copied the definition of xi from
        http://www.math.ubc.ca/~pugh/RiemannZeta/RiemannZetaLong.html
    """
    def gamma(n): return n.gamma()
    def zeta(s): return s.zeta()
    s = pari.pari(s)
    if (s-1).abs() < pari.pari('0.000000000001'):
        return 0.5
    pi = pari.pi()
    w = gamma(s/2 + 1) * (s-1) * pi**(-s/2) * zeta(s)
    return w.python()

def pi(prec=0):
    return pari.pi(prec).python()
    

# doctests
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
